package com.amphinicy.mhp.communication.client;


public interface IController {

  /**
   * Connect to server
   * @throws ControllerException
   */
  void connect()
      throws ControllerException;

  /**
   * Disconnect from server
   * @throws ControllerException
   */
  void disconnect()
      throws ControllerException;

  /**
   * Post data to server
   * @param data ControllerMessage
   * @throws ControllerException
   */
  void post(ControllerMessage data)
      throws ControllerException;

  /**
   * Send data to server, receive some data from server
   * @param data ControllerMessage
   * @throws ControllerException
   * @return ControllerMessage data from server
   */
  ControllerMessage send(ControllerMessage data)
      throws ControllerException;

  /**
   * Method for receiving messages from server.
   */
  void getNextMessages();

  /**
   * Returns status of connection
   * @throws ControllerException
   * @return boolean
   */
  boolean isConnected()
      throws ControllerException;

  /**
   * Add message listener to controller
   * @param l MessageListener
   */
  void addMessageListener(MessageListener l);

  /**
   * Remove message listener from controller
   * @param l MessageListener
   */
  void removeMessageListener(MessageListener l);

  /**
   * Clear all listeners from controller
   */
  void clearMessageListener();

}
