package com.amphinicy.mhp.communication.server;

import java.net.DatagramSocket;
import java.io.IOException;
import java.net.DatagramPacket;
import com.amphinicy.mhp.communication.client.ControllerMessage;
import java.util.Hashtable;
import com.amphinicy.mhp.util.Utils;

public class EchoDatagramSocketServer
    extends Thread {
  public final static int DEFAULT_PORT = 6789;
  private int port;
  private DatagramSocket listen_socket;

  private Hashtable clientTable = new Hashtable();

  // Exit with an error message, when an exception occurs.
  public static void fail(Exception e, String msg) {
    System.err.println(msg + ": " + e);
    System.exit(1);
  }

  // Create a ServerSocket to listen for connections on;  start the thread.
  public EchoDatagramSocketServer(int port) {
    this.port = port;
    try {
      // create a Server socket
      listen_socket = new DatagramSocket(this.port);
      System.out.println("EchoServer: waiting on port " + port);
      this.start();
    }
    catch (IOException e) {
      fail(e, "EchoServer: Exception creating server socket");
    }
  }

  // The body of the server thread.  Loop forever, listening for and
  // accepting connections from clients.  For each connection,
  // create a Connection object to handle communication through the
  // new Socket.
  public void run() {
    byte[] buffer = new byte[65507];
    DatagramPacket incoming;
    while (true) {
      incoming = new DatagramPacket(buffer, buffer.length);
      try {
        listen_socket.receive(incoming);
        respond(listen_socket, incoming);
      }
      catch (IOException e) {
        System.out.println("EchoServer-run() caught " + e);
      }
    }
  }

  public void respond(DatagramSocket ds, DatagramPacket dp) {

    DatagramPacket outgoing;
    String clientName = new String(dp.getAddress().toString() +":"+ dp.getPort());
    ControllerMessage recvMessage = new ControllerMessage();

    try {

      recvMessage.setMessageData(dp.getData());

      Integer clientData = (Integer)clientTable.get(clientName);
      if(clientData == null)
      {
        clientData = new Integer(1);
      }
      else
      {
        clientData = new Integer(clientData.intValue() + 1);
      }
      clientTable.put(clientName, clientData);

      String respData = new String(recvMessage.getPayloadData(), ControllerMessage.ENCODING);
      respData +=  " count=" + clientData + ".";
      ControllerMessage respMessage = new ControllerMessage(
        respData.getBytes(ControllerMessage.ENCODING), ControllerMessage.POST);

      byte[] respByData = respMessage.getMessageData();
      System.out.println("EchoServer-responsd() send size: " + respByData.length);
      System.out.println("EchoServer-responsd() send: " + Utils.byteToHex(respByData));
      outgoing = new DatagramPacket(respByData, respByData.length,
                                    dp.getAddress(), dp.getPort());
      ds.send(outgoing);
    }
    catch (IOException e) {
      System.out.println("EchoServer-responsd() caught " + e);
    }

  }

  // Start up server
  public static void main(String[] args) {
    new EchoDatagramSocketServer(DEFAULT_PORT);
  }
}

