package com.amphinicy.mhp.communication.server;

import java.net.ServerSocket;
import java.io.IOException;
import java.net.Socket;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import com.amphinicy.mhp.communication.client.ControllerMessage;

public class EchoSocketServer
    extends Thread {
  public final static int DEFAULT_PORT = 6789;
  private int port;
  private ServerSocket listen_socket;

  // Exit with an error message, when an exception occurs.
  public static void fail(Exception e, String msg) {
    System.err.println(msg + ": " + e);
    System.exit(1);
  }

  // Create a ServerSocket to listen for connections on;  start the thread.
  public EchoSocketServer(int port) {
    this.port = port;
    try {
      // create a Server socket
      listen_socket = new ServerSocket(port);
      System.out.println("EchoServer: listening on port " + port);
      this.start();
    }
    catch (IOException e) {
      fail(e, "EchoServer: Exception creating server socket");
    }
  }

  // The body of the server thread.  Loop forever, listening for and
  // accepting connections from clients.  For each connection,
  // create a Connection object to handle communication through the
  // new Socket.
  public void run() {
    try {
      while (true) {
        new Connection(listen_socket.accept());
      }
    }
    catch (IOException e) {
      fail(e, "EchoServer: Exception while listening for connections");
    }
  }

  // Start up server
  public static void main(String[] args) {
    new EchoSocketServer(DEFAULT_PORT);
  }
}

// This class is the thread that handles all communication with a client
class Connection
    extends Thread {
  private DataOutputStream out = null;
  private DataInputStream in = null;
  Socket client_socket = null;
  int counter = 0;

  public Connection(Socket client_socket) {
    try {
      // initialize streams to/from the socket
      System.out.println("EchoServer: Connection established");

      this.client_socket = client_socket;
      out = new DataOutputStream(client_socket.getOutputStream());
      in = new DataInputStream(client_socket.
          getInputStream());
      this.start();
    }
    catch (IOException ex) {
      try {
        client_socket.close();
      }
      catch (IOException ex2) {}
      System.err.println("EchoServer: Exception while getting socket streams: " +
                         ex);
      return;
    }
  }

  public void run() {

    try {
      for (; ; ) {
        // read
        ControllerMessage message = new ControllerMessage(in);
        if (message.getType() < 0) {
          System.out.println("EchoServer: client disconnected");
          break;
        }

        // Here the service is provided. In this case it is simply reversing the string that
        // was sent from the Client
        String response = new String(message.getPayloadData(), ControllerMessage.ENCODING);
        System.out.println("EchoServer: received: " + response);

        response += " count=" + (++counter) + ".";
        ControllerMessage responseMessage = null;

        if(message.getType() == message.POST)
        {
          responseMessage = new ControllerMessage(response.getBytes(ControllerMessage.ENCODING), ControllerMessage.POST);
        }
        else
        {
          responseMessage = new ControllerMessage(response.getBytes(ControllerMessage.ENCODING), ControllerMessage.SEND);
        }

        // send answer back to client
        System.out.println("EchoServer: sending: " + response);

        out.write(responseMessage.getMessageData());
      }
    }
    catch (IOException e) {
      System.out.println("EchoServer-run() caught " + e);
    }
    finally {
      try {
        out.close();
        in.close();
        client_socket.close();
      }
      catch (IOException e2) {
        System.out.println("EchoServer-run() caught " + e2);
      }
    }
  }
}
